
# q-expansion of newform 1150.4.a.d, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1150
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-2], [9], [0], [-2], [-52], [-43], [50], [-74], [23], [-7], [-273], [4], [123], [152], [-75], [-86], [-444], [262], [-764], [-21], [-681], [426], [-902], [-1272], [342], [-1426], [1190], [1210], [-1680], [-1030], [2279], [987], [1644], [2189], [-946], [-365], [108], [1415], [-1756], [-2358], [1073], [2868], [332], [2143], [2739], [752], [-1016], [1120], [-2706], [6140], [-6567], [-729], [-2912], [398], [-8131], [1978], [-8459], [-7240], [1319], [1770], [-4144], [6812], [5692], [5267], [-6340], [8794], [8225], [2576], [-596], [-9271], [-8141], [-2130], [2574], [4504], [2740], [-6948], [-1404], [8641], [-1140], [12529], [3252], [2206], [14316], [-7828], [-16039], [-11747], [-2890], [13126], [14481], [-5272], [13466], [-4526], [8795], [-1275], [-9533], [13398], [-8031], [-21184], [21706], [5781], [-7809], [20240], [-7612], [19484], [6614], [639], [829], [20610], [-17240], [-8459], [-17840], [-2534], [5610], [-11948], [-7840], [-2320], [-1864], [-11939], [-10503], [10950], [-3210], [13517], [7494], [-17865], [22364], [7842], [11234], [-17568], [35664], [27914], [39529], [-10062], [25644], [-37368], [105], [-15464], [-35168], [21216], [-9506], [-20550], [-5161], [7866], [22317], [26196], [5886], [32394], [31286], [2913], [15451], [20627], [6966], [-37590], [-27876], [-9471], [-28366], [-7210], [17198], [-51033], [-33328], [-20166], [28629], [-38146], [-44621], [5950], [-40836], [-26874], [21472], [-6286], [41998], [-28739], [55936], [-27347], [-32734], [39656], [28085], [-18522], [-48955], [-13053], [-4898], [-46858], [55075], [-53548], [59087], [-31746], [-41936], [29490], [20662], [-28902], [-7195], [43840], [-1367], [-4136], [704], [4090], [52604], [-47904], [-42748], [-10172], [-33452], [-5557], [74580], [-3720], [13843], [71108], [-65424], [-39457], [-30880], [-28411], [6219], [-86347], [-29607], [-29189], [-90700], [77770], [38632], [53880], [18173], [4485], [70776], [83277], [25515], [-24140], [-83415], [41442], [33033], [18723], [-71792], [5625], [24526], [-7372], [-47114], [-68470], [-69644], [-58414], [-108124], [9498], [-40052], [18812], [101623], [-4171], [-87184], [44915], [-87151], [-6185], [-97086], [-102417], [-23327], [-37514], [298], [7440], [52506], [-108980], [-43554], [49534], [-102090], [-94415], [-4720], [12500], [123319], [-69821], [85840], [66900], [59906], [96586], [-28206], [113562], [21500], [22364], [69218], [-79685], [-115787], [-94970], [89616], [59274], [-99249], [-9639], [50328], [5954], [70632], [-53580], [136215], [107492], [10008], [-152102], [89916], [-20028], [-39698], [-28646], [47708], [55794], [137642], [30942], [11694], [-125478], [-677], [75774], [130630], [-31371], [-99507], [-48213], [39366], [11478], [-96914], [105799], [-72952], [-151819], [-84], [44312], [-169464], [-37234], [118313], [-86738], [109626], [50155], [-117434], [-106162], [-20183], [10051], [9728], [-85241], [37132], [-46414], [-177473], [-63412], [16824], [11628], [143812], [-41953], [-159760], [-52790], [-41335], [-137475], [-202231], [61239], [-134014], [86863], [5912], [-49058], [132539], [-82400], [-85987], [39440], [-4198], [57358], [46400], [39968], [97475], [-130332], [163422], [49420], [44427], [32879], [-170178], [196058], [-163234], [-23874], [-56738], [-131244], [150239], [-48016], [-170152], [96949], [-132317], [-120660], [-56809], [122139], [4405], [-78672], [114434], [7201], [15756], [57745], [-29910], [-27572], [77037], [-120575], [-95712], [-216578], [259020], [-161079], [-100512], [43998], [123404], [29817], [187598], [137011], [132889], [-122006], [5704], [124306], [-122674], [30562], [-235005], [-48316], [-8406], [-208492], [198584], [127362], [-174274], [-66566], [223303], [135239], [-263178], [93154], [-85901], [182019], [179710], [-133751], [-15266], [-292068], [-168967], [-24915], [-21576], [-75946], [-67782], [78213], [-93120], [45400], [32148], [-157568], [60406], [-4176], [-118224], [110244], [108299], [-60431], [-26922], [60026], [7515], [-98881]]
